/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdamgain.h"
#include <gdam/gdamsample.h>
#include <gtk/gtksignal.h>
#include "gdammodel.h"
#include "gdamfilter.h"
#include "gdamutil.h"

static void gdam_gain_class_init(GdamGainClass* klass);

static int gdam_gain_append_start_request(GdamFilter* filter,
				          int new_source_id,
				          GdamRequestBuilder* builder);

static void gdam_gain_init(GdamGain* gain)
{
	gain->volume = 1.0; 
} 

GtkType gdam_gain_get_type()
{
	static GtkType gain_type = 0;
	if (!gain_type) {
		static const GtkTypeInfo gain_info = {
			"GdamGain",
			sizeof(GdamGain),
			sizeof(GdamGainClass),
			(GtkClassInitFunc) gdam_gain_class_init,
			(GtkObjectInitFunc) gdam_gain_init,
			/* reserved 1 */ NULL,
			/* reserved 2 */ NULL,
			(GtkClassInitFunc) NULL
		};
		gain_type = gtk_type_unique(GDAM_TYPE_FILTER, &gain_info);
	}
	return gain_type;
}

static inline void maybe_dispense_request(GdamGain* gain)
{
	GdamArg arg;

	arg.name = "gain";
	arg.type = GDAM_ARG_TYPE_DOUBLE;
	GDAM_ARG_DOUBLE(&arg) = gain->muted ? 0.0 : gain->volume;

	gdam_model_set_args(GDAM_MODEL(gain), 0, 1, &arg);
}

GdamGain* 	gdam_gain_new		()
{
	GdamGain* gain;
	gain = GDAM_GAIN(gtk_type_new(GDAM_TYPE_GAIN));
	return gain;
}
static int gdam_gain_append_start_request(GdamFilter* filter,
				          int new_source_id,
				          GdamRequestBuilder* builder)
{
	GdamGain* gain = GDAM_GAIN(filter);
	GdamRequest* request;
	float volume;
	gtk_object_ref(GTK_OBJECT(gain));
	volume = gain->muted ? 0 : gain->volume;
	request = gdam_request_create_source("GdamGain",
					     gdam_model_got_subsource,
					     gdam_model_didnt_get_subsource,
					     gain,
					     2,
					     "gain",
					     GDAM_ARG_TYPE_DOUBLE,
					     volume,
					     "filtered",
					     GDAM_ARG_TYPE_INT,
					     new_source_id);
	return gdam_request_builder_append_create(builder, request);
}

void gdam_gain_arg_set_volume(GtkObject* object, GdamArg* arg)
{
	GdamGain* gain;
	gain = GDAM_GAIN(object);
	gain->volume = GDAM_ARG_DOUBLE(arg);
	maybe_dispense_request(gain);
}
static void gdam_gain_arg_get_volume(GtkObject* object, GdamArg* arg)
{
	GdamGain* gain;
	gain = GDAM_GAIN(object);
	GDAM_ARG_DOUBLE(arg) = gain->volume;
}

static void gdam_gain_arg_set_muted(GtkObject* object, GdamArg* arg)
{
	GdamGain* gain;
	gain = GDAM_GAIN(object);
	gain->muted = GDAM_ARG_INT(arg) ? 1 : 0;
	maybe_dispense_request(gain);
}
static void gdam_gain_arg_get_muted(GtkObject* object, GdamArg* arg)
{
	GdamGain* gain;
	gain = GDAM_GAIN(object);
	GDAM_ARG_INT(arg) = gain->muted;
}

static void gdam_gain_class_init(GdamGainClass* klass)
{
	GdamFilterClass* filter_class;
	GdamModelClass* model_class;
	GtkObjectClass* object_class;
	filter_class = GDAM_FILTER_CLASS(klass);
	model_class = GDAM_MODEL_CLASS(klass);
	object_class = GTK_OBJECT_CLASS(klass);
	model_class->num_arg_sets = 1;
	filter_class->append_start_request = gdam_gain_append_start_request;
	gdam_arg_register(object_class,
	                  "volume",
			  GDAM_ARG_TYPE_DOUBLE,
			  gdam_gain_arg_set_volume,
			  gdam_gain_arg_get_volume,
			  GDAM_ARG_VITAL);
	gdam_arg_register(object_class,
	                  "muted",
			  GDAM_ARG_TYPE_INT,
			  gdam_gain_arg_set_muted,
			  gdam_gain_arg_get_muted,
			  GDAM_ARG_VITAL);
}



void gdam_gain_set_volume(GdamGain* gain, gdouble volume)
{
	GdamArg arg;
	arg.name = "volume";
	arg.type = GDAM_ARG_TYPE_DOUBLE;
	GDAM_ARG_DOUBLE(&arg) = volume;
	gdam_arg_set_object_arg(GTK_OBJECT(gain), &arg);
}

void gdam_gain_set_muted(GdamGain* gain, gboolean muted)
{
	GdamArg arg;
	arg.name = "muted";
	arg.type = GDAM_ARG_TYPE_INT;
	GDAM_ARG_INT(&arg) = muted;
	gdam_arg_set_object_arg(GTK_OBJECT(gain), &arg);
}
